/*
 *  Interactive disassembler (IDA).
 *  TheHyper VM Processor Module.
 *
 *  (c) 2006 Rolf Rolles
 */

#include "../idaidp.hpp"
#include "ins.hpp"
#include "reg.hpp"
#include "TH.hpp"

static void TouchArg( op_t &x, bool bRead )
{
    switch( x.type )
    {
    case o_near:
    case o_far:
        // mark the reference of a jump / call
        ua_add_cref( 0, x.addr, InstrIsSet(cmd.itype, CF_CALL) ? fl_CN : fl_JN);
        break;

    case o_mem:
        // mark the data reference of an instruction
        ua_add_dref( 0, x.addr, bRead ? dr_R : dr_W );
        break;

    case o_vmmem:
        ea_t blah = get_keyed_address(x.addr);

        if(cmd.itype == TH_getmem)
            ua_add_dref( 0, blah, dr_O );

        else
        if(cmd.itype <= TH_x86call)
            ua_add_cref( 0, blah, InstrIsSet(cmd.itype, CF_CALL) ? fl_CN : fl_JN);
        
        break;
    }
}

int emu( void )
{
  ulong Feature = cmd.get_canon_feature();

  if((Feature & CF_STOP) == 0)
      ua_add_cref( 0, cmd.ea+cmd.size, fl_F );

  if(Feature & CF_USE1)
      TouchArg(cmd.Op1, 0);

  if(Feature & CF_USE2)
      TouchArg(cmd.Op2, 1);

  return 1;
}
